layout(location = 0) out vec4 color;

in vec2 vUV;
in vec4 vClipSpace;

uniform sampler2D uSceneDepth;
uniform vec3 uLightColor;

float near_plane = 0.1;
float far_plane = 1000.0;

float LinearizeDepth(float depth)
{
    float z = depth * 2.0 - 1.0; // Back to NDC 
    return (2.0 * near_plane * far_plane) / (far_plane + near_plane - z * (far_plane - near_plane));	
}

// returns the uv coords in screen space [0,1]
vec2 getScreenSpaceUV()
{
	return (vClipSpace.xy / vClipSpace.w) * 0.5 + 0.5;
}


void main()
{
	vec2 uv = getScreenSpaceUV();
	float depth = LinearizeDepth(texture(uSceneDepth, uv).r);
	float fragDepth = LinearizeDepth(gl_FragCoord.z); 
	float alpha = clamp( (depth - fragDepth) / 4.0, 0.0, 1.0);
	
	float dist = 1.0 - clamp(length(vUV - vec2(0.5)) * 2.5, 0.0, 1.0);
	color = vec4(uLightColor,  dist );
}