layout(location = 0) out vec4 color;

in vec4 vClipSpace;
in vec2 vUV;

uniform sampler2D uDiffuseTexture;
uniform sampler2D uSceneDepth;

layout(std140, row_major) uniform Global
{
	float time;
} global;



float near_plane = 0.1;
float far_plane = 1000.0;

float LinearizeDepth(float depth)
{
    float z = depth * 2.0 - 1.0; // Back to NDC 
    return (2.0 * near_plane * far_plane) / (far_plane + near_plane - z * (far_plane - near_plane));	
}

// returns the uv coords in screen space [0,1]
vec2 getScreenSpaceUV()
{
	return (vClipSpace.xy / vClipSpace.w) * 0.5 + 0.5;
}


void main()
{
	vec2 screenUV = getScreenSpaceUV();
	float depth = LinearizeDepth(texture(uSceneDepth, screenUV).r);
	float fragDepth = LinearizeDepth(gl_FragCoord.z); 
	float alpha = clamp( (depth - fragDepth) / 1.0, 0.0, 1.0);
	
	vec2 texUV = vUV;
	texUV.x = texUV.x + global.time * 0.1;
	
	color = vec4( vec3(0.0549, 0.952, 0.7725) * 10.0, texture(uDiffuseTexture, texUV).r * texUV.y * alpha * 0.1);
}