
layout(location = 0) out vec3 color;

in vec2 uv;

uniform sampler2D uWorldNormal;
uniform sampler2D uAlbedoRoughness;
uniform sampler2D uEmissive;
uniform sampler2D uDepth;

uniform int uDebugMode;
uniform mat4 uProjMatrixInv;
uniform mat4 uViewMatrixInv;

float near_plane = 0.1;
float far_plane = 1000.0;

float LinearizeDepth(float depth)
{
    float z = depth * 2.0 - 1.0; // Back to NDC 
    return (2.0 * near_plane * far_plane) / (far_plane + near_plane - z * (far_plane - near_plane));	
}


// returns the world position form the depth buffer
// https://stackoverflow.com/questions/32227283/getting-world-position-from-depth-buffer-value
vec3 WorldPositionFromDepth(float depth, vec2 uv)
{
	float z = depth * 2.0 - 1.0; // form [0,1] to [-1, 1]
	
	vec4 clipSpace = vec4(uv * 2.0 - 1.0, z, 1.0); // clip space vertex
	vec4 viewSpace = uProjMatrixInv * clipSpace; // transform back to view space
	
	viewSpace /= viewSpace.w;
	
	return (uViewMatrixInv * viewSpace).xyz; // transform back to world position
}

void main()
{
	switch(uDebugMode)
	{
		case 0:
			color = WorldPositionFromDepth(texture2D( uDepth, uv).r, uv);
			color = fract(color);
			break;
		case 1:
			color = texture2D(uWorldNormal, uv).rgb * 0.5 + 0.5;
			break;
		case 2:
			float depth = LinearizeDepth(texture2D( uDepth, uv).r);
			color = vec3(depth / 100);
			break;
		case 3:
			color = texture2D( uAlbedoRoughness, uv).rgb;
			break;
		case 4:
			//color = texture2D( uAlbedoRoughness, uv).aaa;
			color = texture2D(uWorldNormal, uv).www;
			break;
	}

}

