
layout(location = 0) out vec3 color;

in vec2 uv;

uniform sampler2D tex;
uniform sampler2D blur;
uniform float exposure;

layout(std140, row_major) uniform Global
{
	float time;
} global;

// Reinhard tone mappong
vec3 ReinhardTonemap(vec3 hdr)
{	
	vec3 mapped = vec3(1.0) - exp(-hdr * exposure);

	return mapped;
}


vec3 ACESFilm( vec3 hdrcolor )
{
    float a = 2.51f;
    float b = 0.03f;
    float c = 2.43f;
    float d = 0.59f;
    float e = 0.14f;
	hdrcolor *= exposure;
    vec3 color = clamp((hdrcolor*(a*hdrcolor+b))/(hdrcolor*(c*hdrcolor+d)+e), vec3(0.0), vec3(1.0));
	return color;
}


vec3 filmicToneMapping(vec3 color)
{
	color = max(vec3(0.0), color - vec3(0.004));
	color = (color * (6.2 * color + .5)) / (color * (6.2 * color + 1.7) + 0.06);
	return color;
}

vec3 Uncharted2ToneMapping(vec3 color)
{
	float A = 0.22; // Shoulder Strength
	float B = 0.30; // Linear Strength
	float C = 0.10; // Linear Angle
	float D = 0.20; // Toe Strength
	float E = 0.01; // Toe Numerator
	float F = 0.30; // Toe Denominator
	float W = 11.2; // white
	color *= exposure;
	color = ((color * (A * color + C * B) + D * E) / (color * (A * color + B) + D * F)) - E / F;
	float white = ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
	color /= white;

	return color;
}

float near_plane = 0.1;
float far_plane = 100.0;

float LinearizeDepth(float depth)
{
    float z = depth * 2.0 - 1.0; // Back to NDC 
    return (2.0 * near_plane * far_plane) / (far_plane + near_plane - z * (far_plane - near_plane));	
}


void main()
{
	// bloom
    color =  texture2D( tex, uv).rgb * 1 + texture2D( blur, uv).rgb * 1;

	color = Uncharted2ToneMapping(color);

	// gammer correction
	color = pow(color, vec3(1.0 / 2.2));

	//float depthValue = texture2D( tex, uv).r;
	//color = vec3(LinearizeDepth(depthValue) / far_plane);

}