
layout (location = 0) out vec4 fWorldNormal;
layout (location = 1) out vec4 fAlbedoRoughness;
layout (location = 2) out vec3 fEmissive;

in vec2 vUV;
in vec3 vWorldNormal;
in vec3 vWorldPosition;
in mat3 vTBN;

uniform sampler2D uNormalOffset;
uniform samplerCube uCubemap;
uniform float uOpen;
uniform vec3 uCameraWorldPos;

layout(std140, row_major) uniform Global
{
	float time;
} global;

//calculate new world normal from  TBN & normal map
vec3 calcNormal()
{
	vec3 n = texture(uNormalOffset, vUV + vec2( global.time, 0)).rgb;
	//n.g = 1 - n.g;
	n = normalize(n * (255.0/128.0) - 1.0);   
	n = normalize(vTBN * n); 
	return n;
}

#ifdef SHADOW
void main()
{

}
#else
void main()
{
	if(uOpen<0.5)
	{
		discard;
	}

	// calculate normal from normal map
	fWorldNormal.xyz  = calcNormal();
	fWorldNormal.w = 0.0f;
	// sRGB to linear
	fAlbedoRoughness.rgb = vec3(1.0, 0.71, 0.29);

	fAlbedoRoughness.a = 0.1;
	
	vec3 cameraVec = normalize(vWorldPosition - uCameraWorldPos);
	vec3 R = reflect(cameraVec, fWorldNormal.xyz);
	fEmissive.rgb = pow(texture(uCubemap, R).rgb, vec3(2.2)).rgb * 5.0;
}
#endif