
layout(location = 0) in vec4 aPosition;
layout(location = 1) in vec2 aTexCoord;
layout(location = 2) in vec3 aNormal;
layout(location = 3) in vec3 aTangent;

uniform mat4 uView;
uniform mat4 uProjection;
uniform mat4 uModel;

layout(std140, row_major) uniform Global
{
	float time;
} global;


out vec2 vUV;
out vec3 vWorldNormal;
out vec3 vWorldPosition;
out vec3 vWorldTangent;
out mat3 vTBN;

#ifdef SHADOW
void main()
{
	gl_Position = uProjection * uView * uModel * aPosition;
	vUV = aTexCoord;
}
#else
void main()
{
	gl_Position = uProjection * uView * uModel * aPosition;

	vWorldPosition = vec3(uModel * aPosition);

	vWorldNormal = (uModel * vec4(aNormal, 0.0)).xyz;

	mat3 traInvModel = mat3(transpose(inverse(uModel)));
	vWorldNormal = traInvModel * aNormal;
	vWorldTangent = traInvModel * aTangent;
	
	// re-orthogonalize Tangent with respect to N
	vWorldTangent = normalize(vWorldTangent - dot(vWorldTangent, vWorldNormal) * vWorldNormal);
	// then retrieve perpendicular vector B with the cross product of T and N
	vec3 worldBinormal = cross(vWorldNormal, vWorldTangent);

	vTBN = mat3(vWorldTangent, worldBinormal, vWorldNormal);

	vUV = aTexCoord;
}
#endif