
layout (location = 0) out vec4 fWorldNormal;
layout (location = 1) out vec4 fAlbedo;
layout (location = 2) out vec3 fEmissive;

in vec2 vUV;
in vec3 vWorldNormal;
in vec3 vWorldPosition;
in vec3 vLightLocation;

uniform mat4 uModel;
uniform mat4 uView;
uniform mat4 uProjection;
uniform vec3 uCameraWorldPos;
uniform samplerCube uCubemap;


layout(std140, row_major) uniform Global
{
	float time;
} global;


vec3 getModelLocation()
{
	vec3 result;
	result.x = uModel[3][0];
	result.y = uModel[3][1];
	result.z = uModel[3][2];
	return result;
}

float smin(float a, float b, float k)
{
	float h = clamp( 0.5+0.5 *(b-a)/k, 0.0, 1.0);
	return mix(b, a, h) - k * h * (1.0-h);
}

float GetDist(vec3 p)
{
	vec4 modelLocation = vec4(getModelLocation(), 0.0);

	float x = cos(global.time) * 0.5;
	float y = sin(global.time) * (sin(global.time * 2.2) + 1.0) * 0.2 + 0.5;

	vec4 s = modelLocation +  vec4(0, 1.5, 0, 0.5);
	vec4 s2 = modelLocation + vec4(0, x + 1.5, y + 0, (sin(global.time * 2.2) + 1.0) * 0.2 + 0.1);
	vec4 s3 = modelLocation + vec4(0 + y, x + 1.7, 0, (sin(global.time * 2.2 - 1) + 1.0) * 0.2 + 0.1);

	float sphereDist = length(p-s.xyz) - s.w;
	float sphereDist2 = length(p-s2.xyz) - s2.w;
	float sphereDist3 = length(p-s3.xyz) - s3.w;

	return smin(smin(sphereDist, sphereDist2, 0.2), sphereDist3, 0.2);
}

float RayMarch(vec3 ro, vec3 rd)
{
	float d0 = 0;
	for(int i = 0; i < 100; i++)
	{
		vec3 p = ro + d0 * rd;
		float dS = GetDist(p);
		d0 += dS;
		if(dS < 0.01) break;
		if(d0 > 100 ) discard;
	}
	return d0;
}

vec3 GetNormal(vec3 p)
{
	vec2 e = vec2(0.01, 0.0);
	float d = GetDist(p);
	vec3 n = vec3(
		d - GetDist(p - e.xyy),
		d - GetDist(p - e.yxy),
		d - GetDist(p - e.yyx)
	);
	return normalize(n);
}

#ifdef SHADOW
void main()
{
	vec3 cameraVec = normalize(vWorldPosition - vLightLocation);
	vec3 currentPos = vLightLocation;
	
	float d = RayMarch(currentPos, cameraVec);
}
#else
void main()
{
	fAlbedo = vec4(1.0, 0.71, 0.29, 0.3);
	fWorldNormal.xyz = vec3(0.0);
	fEmissive = vec3(0.0);
	
	vec3 cameraVec = normalize(vWorldPosition - uCameraWorldPos);
	vec3 currentPos = uCameraWorldPos;
	
	float d = RayMarch(currentPos, cameraVec);
	//fEmissive = vec3(d / 100.0);
    //if(d == 0.00) fEmissive = vec3(0, 1, 0);
	
	vec3 worldPos = currentPos + cameraVec * d;
	fWorldNormal.xyz = GetNormal( worldPos);
	fWorldNormal.w = 1.0;
	
	
	vec4 Pclip = uProjection* uView * vec4(worldPos, 1.0);
	float ndc_depth = Pclip.z / Pclip.w;
	
	gl_FragDepth = ((gl_DepthRange.diff * ndc_depth) + gl_DepthRange.near + gl_DepthRange.far) / 2.0;
	
	vec3 R = reflect(cameraVec, fWorldNormal.xyz);
	fEmissive.rgb = pow(texture(uCubemap, R).rgb, vec3(2.2)).rgb * fAlbedo.rgb;
}
#endif