
layout(location = 0) in vec4 aPosition;
layout(location = 1) in vec2 aTexCoord;
layout(location = 2) in vec3 aNormal;
layout(location = 3) in vec3 aTangent;
layout(location = 4) in ivec4 aBoneIDs;
layout(location = 5) in vec4 aWeights;

uniform mat4 uView;
uniform mat4 uProjection;
uniform mat4 uModel;

layout(std140, row_major) uniform Global
{
	float time;
} global;


out vec2 vUV;
out vec3 vWorldNormal;
out vec3 vWorldPosition;

uniform mat4 uBones[100];

vec4 getSkinned(vec4 vertex)
{
	return(uBones[aBoneIDs[0]] * vertex) * aWeights[0] + 
	(uBones[aBoneIDs[1]] * vertex) * aWeights[1] + 
	(uBones[aBoneIDs[2]] * vertex) * aWeights[2] + 
	(uBones[aBoneIDs[3]] * vertex) * aWeights[3];
}

#ifdef SHADOW
void main()
{
	gl_Position = uProjection * uView * uModel * getSkinned(aPosition);
	vUV = aTexCoord;
}
#else
void main()
{
	vec4 newPos = getSkinned(aPosition);
	gl_Position = uProjection * uView * uModel * newPos;

	vWorldPosition = vec3(uModel * newPos);

	vWorldNormal = mat3(transpose(inverse(uModel))) * getSkinned(vec4(aNormal, 0.0)).xyz;

	vUV = aTexCoord;
}
#endif