
layout(location = 0) out vec4 color;

in vec3 pos;
in vec3 stars_pos;

uniform sampler2D uDepth;


//Colors for the poles
vec3 colorA = vec3(0.000, 0.455, 0.851);
vec3 colorB = vec3(0.694, 0.051, 0.788);

//Noise generation for the stars
float Hash(float n)
{
	return fract((1.0 + sin(n)) * 415.92653);
}

float Noise3d(vec3 v)
{	
	float a = 200;
	
	//make a larger -> get smaller stars
	float xhash = Hash(round(a*v.x)*37.0);
	float yhash = Hash(round(a*v.y)*57.0);
	float zhash = Hash(round(a*v.z)*67.0);
	
	return fract(xhash + yhash + zhash);
}

// returns the uv coords in screen space [0,1]
vec2 getScreenSpaceUV()
{
	return gl_FragCoord.xy / textureSize( uDepth, 0);
}

void main() {

	vec2 uv = getScreenSpaceUV();
	float depth = texture2D( uDepth, uv).r;

	if( depth != 1)
	{
		discard;
	}

	vec3 stars = vec3(0.0);
	vec3 norm_pos = normalize(pos);
	float threshold = 0.99;
	
	///////////////////Stars///////////////////////
	
	//Generate a random value between 0 and 1
	float star_intensity = Noise3d(normalize(stars_pos));
	
	//Apply a threshold to keep only the brightest areas
	if(star_intensity >= threshold)
	{
		//Compute star intensity
		star_intensity = pow((star_intensity - threshold)/(1.0 - threshold), 6.0);
		stars += vec3(star_intensity);
	}
	////////////////////////////////////////////////
	
	// Mix uses the height norm_pos.y (a value from 0-1) to
    // mix the two colors

	float alpha = (pos.y + 100) / 200;
    color = vec4(mix(colorA, colorB, alpha), 1);
	color += vec4(stars, 1);
}
