layout(location = 0) in vec4 aPosition;
layout(location = 1) in vec2 aTexCoord;
layout(location = 2) in vec3 aNormal;

uniform mat4 uView;
uniform mat4 uProjection;
uniform mat4 uModel;

uniform mat4 uInstances[200];

layout(std140, row_major) uniform Global
{
	float time;
} global;


out vec2 vUV;
out vec3 vWorldNormal;
out vec3 vWorldPosition;

vec4 getWorldPos()
{
	vec3 world_position = vec3(uInstances[gl_InstanceID] * aPosition);

	// calc wind
	vec4 offset = vec4(pow(cos(global.time * 2 + world_position.x / 7), 2) * 0.15, 0, pow(sin(global.time * 2 + world_position.y / 7), 2) * 0.1f, 0);
	offset += vec4(cos(global.time * 0.1 + world_position.x) * 0.25, 0, sin(global.time * 0.1 + world_position.y) * 0.25f, 0);
	
	float weight = clamp(1.0 - aTexCoord.y, 0.0, 1.0);
	return aPosition + offset * weight;
}

#ifdef SHADOW
void main()
{
	gl_Position = uProjection * uView * uInstances[gl_InstanceID]  * getWorldPos();
}
#else
void main()
{
	vec4 newPos = getWorldPos();
	gl_Position = uProjection * uView * uInstances[gl_InstanceID]  * newPos;

	vWorldPosition = vec3(uInstances[gl_InstanceID]  * newPos);

	vWorldNormal = vec3(0, 1, 0);

	vUV = aTexCoord;
}
#endif
