
layout(location = 0) out vec4 color;

in vec2 vertex_uv;
in vec3 world_normal;
in vec3 world_position;
in vec4 light_position;

uniform sampler2D tex;
uniform sampler2D shadowMap;
uniform samplerCube cubemap;

uniform vec3 u_camera_world_pos;
uniform vec3 u_color;
uniform vec3 u_playerLocation;

//Colors for the poles
vec4 colorA = vec4(0.000, 0.455, 0.851, 1.0);
vec4 colorB = vec4(0.694, 0.051, 0.788, 1.0);


layout(std140, row_major) uniform Global
{
	float time;
} global;

layout(std140, shared) uniform RenderView
{
	vec3 cameraLocation;
	vec4 sizeAndInfSize;
} renderView;

float playerAO()
{
	vec3 dist = (u_playerLocation - world_position + vec3( 0, 0.9, 0)) / vec3(0.75, 2.0, 0.75);
	
	float a = dot(dist, dist);
	a = a * a * a;
	float ra = inversesqrt(a);
	float b = clamp( ra, 0.0, 1.0);
	b = b * b;
	return (1.0 - b) ;
}

float GetDirectionalLightOcclusion()
{
	// transform to normalized space
	vec3 projLightCoords = light_position.xyz / light_position.w;
	projLightCoords = (projLightCoords * 0.5) + 0.5; // map from [-1, 1] to [0,1]

	if(projLightCoords.z > 1.0)
	{
		return 0.0f;
	}

	float bias = 0.005;

	vec2 shadowMapTexelSize = 1.0 / textureSize(shadowMap, 0);
	float current = projLightCoords.z;

	float shadowing = 0.0f;
	for(int x = -1; x <= 1; x++)
	{
		for(int y = -1; y <= 1; y++)
		{
			float depth = texture(shadowMap, projLightCoords.xy + vec2( x, y) * shadowMapTexelSize ).r;
			shadowing += ((current - bias) > depth) ? 1.0 : 0.0;
		}
	}

	return shadowing / 9.0;
}

vec4 DirectionalLight( vec3 direction, vec3 color)
{
	direction = normalize(-direction);
	
	float diffuse = max(0.0f, dot(world_normal, direction));
	
	float specular_strength = 1.0f;
	vec3 reflect_dir = reflect(-direction, normalize(world_normal));
	vec3 eye = normalize(renderView.cameraLocation - world_position);
	float specular = specular_strength * pow(max(0.0f ,dot(eye, reflect_dir)), 32.0f); //32.0f->shininess factor

	float shadow = GetDirectionalLightOcclusion();

	return vec4(specular + diffuse) * vec4(color, 1.0) * ( 1.0 - shadow);
}

vec4 calcAmbientLighting(vec3 normal)
{
	float alpha = dot(world_normal, vec3( 0,1, 0));
	alpha = (alpha + 1) * 0.5f;
	return mix(colorA, colorB, alpha) * 0.1;

//	return vec4(textureLod( cubemap , normal, 10).rgb, 1.0);
}

void main()
{
	//float lp = gl_FragCoord.y / 500.0;
	vec4 texColor = texture( tex, vertex_uv);
	float a = (sin(global.time) + 1) * 0.5f;
	color = DirectionalLight( vec3(-0.8f,-1.0f,0.0f), vec3(3.140) ) * playerAO(); 
	color += calcAmbientLighting(world_normal) * 1.0;
	color *= vec4(u_color, 1.0);
	//color = vec4(world_normal * 0.5f + 0.5, 1);
}