layout(location = 0) out vec3 color;

in vec2 uv;

uniform sampler2D uHDRTexture;
uniform float uExposure;
uniform float uBrightness;
uniform int uMode;

// Reinhard tone mappong
vec3 ReinhardTonemap(vec3 hdr)
{	
	vec3 mapped = vec3(1.0) - exp(-hdr * uExposure);
	return mapped;
}

// ACES Tone mapper
vec3 ACESFilm( vec3 hdrcolor )
{
    float a = 2.51f;
    float b = 0.03f;
    float c = 2.43f;
    float d = 0.59f;
    float e = 0.14f;
	hdrcolor *= uExposure;
    vec3 color = clamp((hdrcolor*(a*hdrcolor+b))/(hdrcolor*(c*hdrcolor+d)+e), vec3(0.0), vec3(1.0));
	return color;
}

// filmic tone mapper
vec3 filmicToneMapping(vec3 color)
{
	color = max(vec3(0.0), color - vec3(0.004));
	color = (color * (6.2 * color + .5)) / (color * (6.2 * color + 1.7) + 0.06);
	return color;
}

// uncharted 2 tone mapper
vec3 Uncharted2ToneMapping(vec3 color)
{
	float A = 0.22; // Shoulder Strength
	float B = 0.30; // Linear Strength
	float C = 0.10; // Linear Angle
	float D = 0.20; // Toe Strength
	float E = 0.01; // Toe Numerator
	float F = 0.30; // Toe Denominator
	float W = 11.2; // white

	color *= uExposure;
	color = ((color * (A * color + C * B) + D * E) / (color * (A * color + B) + D * F)) - E / F;
	float white = ((W * (A * W + C * B) + D * E) / (W * (A * W + B) + D * F)) - E / F;
	color /= white;

	return color;
}

void main()
{	
	// sample hdr color;
	color = texture2D( uHDRTexture, uv).rgb;

	// apply tone mapper
	
	switch(uMode)
	{
		case 0:
			color = ReinhardTonemap(color);
			break;
		case 1:
			color = ACESFilm(color);
			break;
		case 2:
			color = filmicToneMapping(color);
			break;
		case 3:
			color = Uncharted2ToneMapping(color);
			break;
	}
	
	// gammer correction
	color = pow(color, vec3(1.0 / 2.2));
	color = color * uBrightness;
}